/*
 * The simplest mpeg encoder (well, it was the simplest!)
 * Copyright (c) 2000,2001 Gerard Lantau.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "avcodec.h"
#include "dsputil.h"
#include "mpegvideo.h"
#include <winsock.h>
#define EDGE_WIDTH 16
void j_rev_dct_ARM (DCTELEM *data);

const UINT8 zigzag_direct[64] = {
    0, 1, 8, 16, 9, 2, 3, 10,
    17, 24, 32, 25, 18, 11, 4, 5,
    12, 19, 26, 33, 40, 48, 41, 34,
    27, 20, 13, 6, 7, 14, 21, 28,
    35, 42, 49, 56, 57, 50, 43, 36,
    29, 22, 15, 23, 30, 37, 44, 51,
    58, 59, 52, 45, 38, 31, 39, 46,
    53, 60, 61, 54, 47, 55, 62, 63
};

const UINT8 default_intra_matrix[64] = {
	8, 16, 19, 22, 26, 27, 29, 34,
	16, 16, 22, 24, 27, 29, 34, 37,
	19, 22, 26, 27, 29, 34, 34, 38,
	22, 22, 26, 27, 29, 34, 37, 40,
	22, 26, 27, 29, 32, 35, 40, 48,
	26, 27, 29, 32, 35, 40, 48, 58,
	26, 27, 29, 34, 38, 46, 56, 69,
	27, 29, 35, 38, 46, 56, 69, 83
};

const UINT8 default_non_intra_matrix[64] = {
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
};
int get_rl_index(const RLTable *rl, int last, int run, int level)
{
    int index;
    index = rl->index_run[last][run];
    if (index >= rl->n)
        return rl->n;
    if (level > rl->max_level[last][run])
        return rl->n;
    return index + level - 1;
}

/* enable all paranoid tests for rounding, overflows, etc... */
//#define PARANOID

//#define DEBUG

/* for jpeg fast DCT */
#define CONST_BITS 14

static const unsigned short aanscales[64] = {
    /* precomputed values scaled up by 14 bits */
    16384, 22725, 21407, 19266, 16384, 12873,  8867,  4520,
    22725, 31521, 29692, 26722, 22725, 17855, 12299,  6270,
    21407, 29692, 27969, 25172, 21407, 16819, 11585,  5906,
    19266, 26722, 25172, 22654, 19266, 15137, 10426,  5315,
    16384, 22725, 21407, 19266, 16384, 12873,  8867,  4520,
    12873, 17855, 16819, 15137, 12873, 10114,  6967,  3552,
    8867, 12299, 11585, 10426,  8867,  6967,  4799,  2446,
    4520,  6270,  5906,  5315,  4520,  3552,  2446,  1247
};

static UINT8 h263_chroma_roundtab[16] = {
    0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2,
};

static void rate_control_init(MpegEncContext *s);
static int rate_estimate_qscale(MpegEncContext *s);
static void dct_unquantize(MpegEncContext *s, DCTELEM *block, int n, int qscale);

/* default motion estimation */
int motion_estimation_method = ME_LOG;

/* XXX: should use variable shift ? */
#define QMAT_SHIFT_MMX 19
#define QMAT_SHIFT 25
#define W1 2841                 /* 2048*sqrt(2)*cos(1*pi/16) */
#define W2 2676                 /* 2048*sqrt(2)*cos(2*pi/16) */
#define W3 2408                 /* 2048*sqrt(2)*cos(3*pi/16) */
#define W5 1609                 /* 2048*sqrt(2)*cos(5*pi/16) */
#define W6 1108                 /* 2048*sqrt(2)*cos(6*pi/16) */
#define W7 565                  /* 2048*sqrt(2)*cos(7*pi/16) */

#define W1_minus_W7 2276
#define W1_plus_W7 3406
#define W3_minus_W5 799
#define W3_plus_W5 4017
#define W2_minus_W6 1568
#define W2_plus_W6 3784

/*
 * Perform the inverse DCT on one block of coefficients.
 */

#if (defined(MIPS)&&(_WIN32_WCE>=300))
//#ifdef MIPSX

//#ifdef GAPI
//it finally uses 64bits MIPSIII instructions!!
static void clearblock (DCTELEM *psblock) 
{ 
__asm ("addi $2,$0,2;" 
"comienzo:sdr $0,0($4);" 
"sdr $0,8($4);" 
"sdr $0,16($4);" 
"sdr $0,24($4);" 
"sdr $0,32($4);" 
"sdr $0,40($4);" 
"sdr $0,48($4);" 
"sdr $0,56($4);" 
"addi $4,$4,64;" 
"addi $2,$2,-1;" 
"bgtz $2,comienzo;",psblock); 
} 
/*#else
static void clearblock (idct_block_t *psblock) 
{ 

__asm ("addi $2,$0,2;" 
"comienzo:sw $0,0($4);" 
"sw $0,4($4);" 
"sw $0,8($4);" 
"sw $0,12($4);" 
"sw $0,16($4);" 
"sw $0,20($4);" 
"sw $0,24($4);" 
"sw $0,28($4);" 
"sw $0,32($4);" 
"sw $0,36($4);" 
"sw $0,40($4);" 
"sw $0,44($4);" 
"sw $0,48($4);" 
"sw $0,52($4);" 
"sw $0,56($4);" 
"sw $0,60($4);" 
"addi $4,$4,64;" 
"addi $2,$2,-1;" 
"bgtz $2,comienzo;",psblock); 
} 
#endif*/
#else

static __inline void clearblock (DCTELEM *psblock)
{
	int i;	
	__int64* t=(__int64*) psblock;
	for (i=16; i!=0;i--)
	{
		psblock++[0]=0;	
	}
}
#endif
//#endif

#if (defined(MIPS)&&(_WIN32_WCE>=300))


//Inverse Discrete Cosinus Transform(IDCT)
//MIPS ASM - MACC VERSION 
//v0.1  => first working version. No optimizations at all 
//v0.2a => its macc optimized.
//v0.3	=> 8 bits destination buffer-bypass buffer conversions.
//v0.4	=> Merged both idctrows functions of v0.3

static void idctrow_special (DCTELEM *blk, UINT8 *destU8,int flag )
{
  __asm("add	$24,$5,$0;"
		"add	$25,$6,$0;"
		"lh		$5,0($4);"		//x0 
		"lh		$9,2($4);"		//x4 
		"lh		$8,4($4);"		//x3 
		"lh		$12,6($4);"		//x7 
		"lh		$6,8($4);"		//x1 
		"lh		$11,10($4);"	//x6 
		"lh		$7,12($4);"		//x2 
		"lh		$10,14($4);"	//x5 

		"sll	$6,$6,8;"		// x1<< 

		"or		$2,$7,$6;" 
		"or		$2,$2,$8;" 
		"or		$2,$2,$9;" 
		"or		$2,$2,$10;" 
		"or		$2,$2,$11;" 
		"or		$2,$2,$12;"		//shortcut 
		 
		"bne $2,$0,idct_row_a;"	//if some values arent 0, when cant shorcut 
			"addi	$5,$5,32;"
			"sra	$5,$5,6;"
			"bne	$25,$0,add_eq;"
			
			"srl	$15,$5,16;"	//if less than 0, then 0
			"srlv	$5,$5,$15;"

			"sll	$15,$5,23;"	//if bigger than 255:255
			"sra	$15,$15,31;"
			"or		$5,$5,$15;"
			
			"andi	$5,$5,0x00FF;"
			"sll	$14,$5,8;"
			"or		$15,$5,$14;"
			"sll	$14,$5,16;"
			"or		$15,$15,$14;"
			"sll	$14,$5,24;"
			"or		$5,$15,$14;"
			
			"sw		$5,0($24);"	//guardamos x0 
			"sw		$5,4($24);" 
			"jr		$31;");
						
__asm(		"add_eq:lbu	$6,0($24);"	//we load older values
			"lbu	$7,1($24);"	//
			"lbu	$8,2($24);"	//	
			"lbu	$9,3($24);"	//
			"lbu	$10,4($24);"//
			"lbu	$11,5($24);"//
			"lbu	$12,6($24);"//
			"lbu	$13,7($24);"//

			"add	$14,$5,$6;"

			"srl	$15,$14,16;"	//if less than 0, then 0
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	//if bigger than 255:255
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,0($24);"	//and we save it

			"add	$14,$5,$7;"		//we do the same with next ones

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,1($24);"	
			
			"add	$14,$5,$8;"		

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,2($24);"

			"add	$14,$5,$9;"		

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,3($24);"
			);

__asm(		"add	$14,$5,$10;"		

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,4($24);"

			"add	$14,$5,$11;"		

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,5($24);"

			"add	$14,$5,$12;"		

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,6($24);"

			"add	$14,$5,$13;"		

			"srl	$15,$14,16;"	
			"srlv	$14,$14,$15;"

			"sll	$15,$14,23;"	
			"sra	$15,$15,31;"
			"or		$14,$14,$15;"
			"sb		$14,7($24);"//and finish

			"jr		$31;",blk); 
	
__asm  ("idct_row_a:add	$13,$9,$10;"//x8=x4+x5
		"li		$14,565;"		//W7
		"mult	$14,$13;"		//W7*x8
		"li		$15,2276;"		//W1_minus_W7 
		"macc	$0,$15,$9;"		//x4=x(W1-W7)*x4+W7*8
		"mflo	$9;"

		"sll	$5,$5,8;"		// x0<<8)
		
		"mult	$14,$13;"		//W7*x8
		"li		$15,-3406;"		//-W1_plus_W7
		"macc	$0,$15,$10;"	//-((W1+W7)*x5)+W7*x8
		"mflo	$10;"
		
		"li		$14,2408;"		//W3
		"add	$13,$11,$12;"	// x6+x7
		"mult	$14,$13;"		//W3*(x6+x7)

		"li		$15,-799;"		//-W3_minus_W5
		"macc	$0,$15,$11;"	//-((W3-W5)*x6)+W3*(x6+x7)
		"mflo	$11;"

		"add	$5,$5,8192;"	//x0+=128

		"mult	$14,$13;"
		"li		$14,-4017;"
		"macc	$0,$12,$14;"
		"mflo	$12;"
		);		

__asm (	"add	$13,$5,$6;" 
		"sub	$5,$5,$6;"		//x0 -=x1 
		
		"add	$6,$7,$8;"		//x1=(x2+x3)
		"li		$15,1108;"		//W6 
		"mult	$15,$6;"		//W6*(x3+x2) 
		
		"li		$14,-3784;"		//-W2_plus_W6 
		"macc	$0,$14,$7;"		//x1+(-W2_plus_W6*x2) 
		"mflo	$7;"
		"mult	$15,$6;"		//W6*(x3+x2)
		"li		$14,1568;"		//W2_minus_W6 
		"macc	$0,$14,$8;"		//W2_minus_W6*x3 
		"mflo	$8;" 

		"addi	$9,$9,4;"
		"addi	$11,$11,4;"
		"addi	$10,$10,4;"
		"addi	$12,$12,4;"

		"sra	$9,$9,3;"
		"sra	$10,$10,3;"
		"sra	$11,$11,3;"
		"sra	$12,$12,3;"
		
		"add	$6,$9,$11;"		//x1=x4+x6 
		"sub	$9,$9,$11;"		//x4=x4-x6 
		"add	$11,$10,$12;"	//x6=x5+x7 
		"sub	$10,$10,$12;"	//x5=x5-x7 
		
); 

__asm (	"addi	$7,$7,4;"
		"addi	$8,$8,4;"
		"sra	$7,$7,3;"
		"sra	$8,$8,3;"
		"add	$12,$13,$8;"	//x7=x8+x3 
		"sub	$13,$13,$8;"	//x8-=x3 
		"add	$8,$5,$7;"		//x3=x0+x2 
		"sub	$5,$5,$7;"		//x0-=x2 

		"li		$14,181;" 
		"add	$7,$9,$10;"		//x4+x5 
		"mult	$14,$7;"		// 
		"mflo	$7;" 
		"addi	$7,$7,128;" 
		"sra	$7,$7,8;" 

		"sub	$9,$9,$10;" 
		"mult	$14,$9;" 
		"mflo	$9;" 
		"addi	$9,$9,128;" 
		"sra	$9,$9,8;"); 

//Fourth Stage 


__asm (	"bne	$25,$0,add;"
		"add	$15,$12,$6;"	//x7+x1 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//if less than 0, then 0
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,0($24);" 

		"add	$15,$7,$8;"		//x3+x2 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,1($24);" 

		"add	$15,$5,$9;"		//x0+x4 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,2($24);" 

		"add	$15,$13,$11;"	//x8+x6 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,3($24);" 

		"sub	$15,$13,$11;" 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,4($24);" 

		"sub	$15,$5,$9;" 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,5($24);" 

		"sub	$15,$8,$7;" 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,6($24);" 

		"sub	$15,$12,$6;" 
		"sra	$15,$15,14;" 
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,7($24);"
		"jr		$31;");

__asm (	"add:add	$15,$12,$6;"	//x7+x1 
		"lbu	$2,0($24);"
		"sra	$15,$15,14;" 
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//if less than 0, then 0
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,0($24);" 

		"add	$15,$7,$8;"		//x3+x2 
		"lbu	$2,1($24);"
		"sra	$15,$15,14;" 
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,1($24);" 

		"add	$15,$5,$9;"		//x0+x4 
		"lbu	$2,2($24);"
		"sra	$15,$15,14;"
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,2($24);" 

		"add	$15,$13,$11;"	//x8+x6 
		"lbu	$2,3($24);"
		"sra	$15,$15,14;"
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,3($24);"
		);

__asm(	"sub	$15,$13,$11;"
		"lbu	$2,4($24);"
		"sra	$15,$15,14;"
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,4($24);" 

		"sub	$15,$5,$9;" 
		"lbu	$2,5($24);"
		"sra	$15,$15,14;"
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,5($24);" 

		"sub	$15,$8,$7;" 
		"lbu	$2,6($24);"
		"sra	$15,$15,14;"
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,6($24);" 

		"sub	$15,$12,$6;"
		"lbu	$2,7($24);"
		"sra	$15,$15,14;" 
		"add	$15,$15,$2;"
		"srl	$14,$15,16;"	//
		"srlv	$15,$15,$14;"	//
		"sll	$14,$15,23;"	//if bigger than 255:255
		"sra	$14,$14,31;"	//
		"or		$15,$15,$14;"	//
		"sb		$15,7($24);"
		); 
}

static void idctcol (DCTELEM *blk) 
{ 

__asm (	"lh		$5,0($4);"//x0 
		"lh		$9,16($4);"//x4 
		"lh		$8,32($4);"//x3 
		"lh		$12,48($4);"//x7 
		"lh		$6,64($4);"//x1 
		"lh		$11,80($4);"//x6 
		"lh		$7,96($4);"//x2 
		"lh		$10,112($4);"//x5 

		"or		$2,$7,$6;" 
		"or		$2,$2,$8;" 
		"or		$2,$2,$9;" 
		"or		$2,$2,$10;" 
		"or		$2,$2,$11;" 
		"or		$2,$2,$12;"//shortcut 

		"sll	$6,$6,11;"// x1<<11 
		
		"bne $2,$0,idct_estandar;"//si no es 0 no podemos atajar 
			"sll	$5,$5,3;"//x0<<3 
			"beq	$5,$0,final;"
			
			"sh		$5,0($4);"//guardamos x0 
			"sh		$5,16($4);" 
			"sh		$5,32($4);"
			"sh		$5,48($4);"
			"sh		$5,64($4);" 
			"sh		$5,80($4);"
			"sh		$5,96($4);"
			"sh		$5,112($4);" 
			"final:jr		$31;",blk); 
	
__asm  ("idct_estandar:add	$13,$9,$10;"//x8=x4+x5
		"li		$14,565;"//W7
		"mult	$14,$13;"//W7*x8
		"li		$15,2276;"//W1_minus_W7 
		"macc	$0,$15,$9;"//x4=x(W1-W7)*x4+W7*8
		"mflo	$9;"

		"sll	$5,$5,11;"// x0<<11
		
		"mult	$14,$13;"//W7*x8
		"li		$15,-3406;"	//-W1_plus_W7
		"macc	$0,$15,$10;"//-((W1+W7)*x5)+W7*x8
		"mflo	$10;"
		
		"li		$14,2408;"//W3
		"add	$13,$11,$12;"// x6+x7
		"mult	$14,$13;"//W3*(x6+x7)

		"li		$15,-799;"//-W3_minus_W5
		"macc	$0,$15,$11;"//-((W3-W5)*x6)+W3*(x6+x7)
		"mflo	$11;"

		"add	$5,$5,128;"//x0+=128

		"mult	$14,$13;"
		"li		$14,-4017;"
		"macc	$0,$12,$14;"
		"mflo	$12;"
		);		

__asm (	"add	$13,$5,$6;" 
		"sub	$5,$5,$6;"//x0 -=x1 
		
		"add	$6,$7,$8;"//x1=(x2+x3)
		"li		$15,1108;"//W6 
		"mult	$15,$6;"//W6*(x3+x2) 
		
		"li		$14,-3784;" //-W2_plus_W6 
		"macc	$0,$14,$7;" //x1+(-W2_plus_W6*x2) 
		"mflo	$7;"
	
		"mult	$15,$6;"//W6*(x3+x2)
		"li		$14,1568;"//W2_minus_W6 
		"macc	$0,$14,$8;"//W2_minus_W6*x3 
		"mflo	$8;" 

		"add	$6,$9,$11;"//x1=x4+x6 
		"sub	$9,$9,$11;"//x4=x4-x6 
		"add	$11,$10,$12;"//x6=x5+x7 
		"sub	$10,$10,$12;"//x5=x5-x7 
		
); 

__asm (	"add	$12,$13,$8;"//x7=x8+x3 
		"sub	$13,$13,$8;"//x8-=x3 
		"add	$8,$5,$7;"//x3=x0+x2 
		"sub	$5,$5,$7;"//x0-=x2 

		"li		$14,181;" 
		"add	$7,$9,$10;"//x4+x5 
		"mult	$14,$7;"// 
		"mflo	$7;" 
		"addi	$7,$7,128;" 
		"sra	$7,$7,8;" 

		"sub	$9,$9,$10;" 
		"mult	$14,$9;" 
		"mflo	$9;" 
		"addi	$9,$9,128;" 
		"sra	$9,$9,8;"); 

//Fourth Stage 

__asm (	"add	$24,$12,$6;"//x7+x1 
		"sra	$24,$24,8;" 
		"sh		$24,0($4);" 

		"add	$24,$7,$8;"//x3+x2 
		"sra	$24,$24,8;" 
		"sh		$24,16($4);" 

		"add	$24,$5,$9;"//x0+x4 
		"sra	$24,$24,8;" 
		"sh		$24,32($4);" 

		"add	$24,$13,$11;"//x8+x6 
		"sra	$24,$24,8;" 
		"sh		$24,48($4);" 

		"sub	$24,$13,$11;" 
		"sra	$24,$24,8;" 
		"sh		$24,64($4);" 

		"sub	$24,$5,$9;" 
		"sra	$24,$24,8;" 
		"sh		$24,80($4);" 

		"sub	$24,$8,$7;" 
		"sra	$24,$24,8;" 
		"sh		$24,96($4);" 

		"sub	$24,$12,$6;" 
		"sra	$24,$24,8;" 
		"sh		$24,112($4);"); 
} 

#else

static void idctcol (DCTELEM *blk)
{
  int x0, x1, x2, x3, x4, x5, x6, x7, x8;

	x4 = blk[8];
	x3 = blk[16];
	x7 = blk[24];
	x1 = blk[32] << 11;
	x6 = blk[40];
	x2 = blk[48];	
	x5 = blk[56];
	
  if (!((x1) | (x2) | (x3) | (x4) | (x5) | (x6) | (x7)))
  {
    blk[0] = blk[8] = blk[16] = blk[24] = blk[32] = blk[40] = blk[48] = blk[56] = blk[0] << 3;
    return;
  }
  x0 = (blk[0] << 11) + 128;    
  x8 = W7 * (x4 + x5);
  x4 = x8 + (W1_minus_W7) * x4;
  x5 = x8 - (W1_plus_W7) * x5;
  x8 = W3 * (x6 + x7);
  x6 = x8 - (W3_minus_W5) * x6;
  x7 = x8 - (W3_plus_W5) * x7;

  x8 = x0 + x1;
  x0 -= x1;
  x1 = W6 * (x3 + x2);
  x2 = x1 - (W2_plus_W6) * x2;
  x3 = x1 + (W2_minus_W6) * x3;
  x1 = x4 + x6;
  x4 -= x6;
  x6 = x5 + x7;
  x5 -= x7;

  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181 * (x4 + x5) + 128) >> 8;
  x4 = (181 * (x4 - x5) + 128) >> 8;

  blk[0] = (DCTELEM)((x7 + x1) >> 8);
  blk[8] = (DCTELEM)((x3 + x2) >> 8);
  blk[16] = (DCTELEM)((x0 + x4) >> 8);
  blk[24] = (DCTELEM)((x8 + x6) >> 8);
  blk[32] = (DCTELEM)((x8 - x6) >> 8);
  blk[40] = (DCTELEM)((x0 - x4) >> 8);
  blk[48] = (DCTELEM)((x3 - x2) >> 8);
  blk[56] = (DCTELEM)((x7 - x1) >> 8);

}
static __inline void idctrow_special (DCTELEM *blk, UINT8 *destU8,int flag)
{
  
  int x0, x1, x2, x3, x4, x5, x6, x7, x8;
  int temp;
	x4 = blk[1];
  	x3 = blk[2];
  	x7 = blk[3];
	x1 = (blk[4] << 8);
	x6 = blk[5];
	x2 = blk[6];
	x5 = blk[7];
	
  if (!((x1) | (x2) | (x3) | (x4) | (x5) | (x6) | (x7)))
  {
    
	temp =  (blk[0] + 32) >> 6; 
	
	if (flag==1) {
		destU8[0] = (temp+destU8[0])>255?255:(temp+destU8[0])<0?0:(temp+destU8[0]);
		destU8[1] = (temp+destU8[1])>255?255:(temp+destU8[1])<0?0:(temp+destU8[1]);
		destU8[2] = (temp+destU8[2])>255?255:(temp+destU8[2])<0?0:(temp+destU8[2]);
		destU8[3] = (temp+destU8[3])>255?255:(temp+destU8[3])<0?0:(temp+destU8[3]);
		destU8[4] = (temp+destU8[4])>255?255:(temp+destU8[4])<0?0:(temp+destU8[4]);
		destU8[5] = (temp+destU8[5])>255?255:(temp+destU8[5])<0?0:(temp+destU8[5]);
		destU8[6] = (temp+destU8[6])>255?255:(temp+destU8[6])<0?0:(temp+destU8[6]);
		destU8[7] = (temp+destU8[7])>255?255:(temp+destU8[7])<0?0:(temp+destU8[7]);
	}
	else	{
		temp=temp>255? 255: temp<0?0:temp;
		destU8[0]=destU8[1]=destU8[2]=destU8[3]=destU8[4]=destU8[5]=destU8[6]=destU8[7]=temp;	
	}
   
	return;
  }
  x0 = (blk[0] << 8) + 8192;

  x8 = W7 * (x4 + x5) + 4;
  x4 = (x8 + (W1_minus_W7) * x4) >> 3;
  x5 = (x8 - (W1_plus_W7) * x5) >> 3;
  x8 = W3 * (x6 + x7) + 4;
  x6 = (x8 - (W3_minus_W5) * x6) >> 3;
  x7 = (x8 - (W3_plus_W5) * x7) >> 3;

  x8 = x0 + x1;
  x0 -= x1;
  x1 = W6 * (x3 + x2) + 4;
  x2 = (x1 - (W2_plus_W6) * x2) >> 3;
  x3 = (x1 + (W2_minus_W6) * x3) >> 3;
  x1 = x4 + x6;
  x4 -= x6;
  x6 = x5 + x7;
  x5 -= x7;

  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181 * (x4 + x5) + 128) >> 8;
  x4 = (181 * (x4 - x5) + 128) >> 8;
  if (flag ==1){
	  destU8[0] = (((x7 + x1) >> 14)+destU8[0])>255? 255: (((x7 + x1) >> 14)+destU8[0])<0 ? 0:(((x7 + x1) >> 14)+destU8[0]);
	  destU8[1] = (((x3 + x2) >> 14)+destU8[1])>255? 255: (((x3 + x2) >> 14)+destU8[1])<0 ? 0:(((x3 + x2) >> 14)+destU8[1]);
	  destU8[2] = (((x0 + x4) >> 14)+destU8[2])>255? 255: (((x0 + x4) >> 14)+destU8[2])<0 ? 0:(((x0 + x4) >> 14)+destU8[2]);
	  destU8[3] = (((x8 + x6) >> 14)+destU8[3])>255? 255: (((x8 + x6) >> 14)+destU8[3])<0 ? 0:(((x8 + x6) >> 14)+destU8[3]);
	  destU8[4] = (((x8 - x6) >> 14)+destU8[4])>255? 255: (((x8 - x6) >> 14)+destU8[4])<0 ? 0:(((x8 - x6) >> 14)+destU8[4]);
	  destU8[5] = (((x0 - x4) >> 14)+destU8[5])>255? 255: (((x0 - x4) >> 14)+destU8[5])<0 ? 0:(((x0 - x4) >> 14)+destU8[5]);
	  destU8[6] = (((x3 - x2) >> 14)+destU8[6])>255? 255: (((x3 - x2) >> 14)+destU8[6])<0 ? 0:(((x3 - x2) >> 14)+destU8[6]);
	  destU8[7] = (((x7 - x1) >> 14)+destU8[7])>255? 255: (((x7 - x1) >> 14)+destU8[7])<0 ? 0:(((x7 - x1) >> 14)+destU8[7]);}
  else{
	  destU8[0] = ((x7 + x1) >> 14)>255? 255: ((x7 + x1) >> 14)<0 ? 0:((x7 + x1) >> 14);
	  destU8[1] = ((x3 + x2) >> 14)>255? 255: ((x3 + x2) >> 14)<0 ? 0:((x3 + x2) >> 14);
	  destU8[2] = ((x0 + x4) >> 14)>255? 255: ((x0 + x4) >> 14)<0 ? 0:((x0 + x4) >> 14);
	  destU8[3] = ((x8 + x6) >> 14)>255? 255: ((x8 + x6) >> 14)<0 ? 0:((x8 + x6) >> 14);
	  destU8[4] = ((x8 - x6) >> 14)>255? 255: ((x8 - x6) >> 14)<0 ? 0:((x8 - x6) >> 14);
	  destU8[5] = ((x0 - x4) >> 14)>255? 255: ((x0 - x4) >> 14)<0 ? 0:((x0 - x4) >> 14);
	  destU8[6] = ((x3 - x2) >> 14)>255? 255: ((x3 - x2) >> 14)<0 ? 0:((x3 - x2) >> 14);
	  destU8[7] = ((x7 - x1) >> 14)>255? 255: ((x7 - x1) >> 14)<0 ? 0:((x7 - x1) >> 14);
	  }
 }

#endif
/* two dimensional inverse discrete cosine transform */ 
static __inline void idct_special (DCTELEM *block, UINT8 *destU8, int stride, int flag)
{

	idctcol(block);
	idctcol(block+1);
	idctcol(block+2);
	idctcol(block+3);
	idctcol(block+4);
	idctcol(block+5);
	idctcol(block+6);
	idctcol(block+7);	
	
	idctrow_special(block,destU8,flag);
	destU8+=stride;
	idctrow_special(block+8,destU8,flag);
	destU8+=stride;
	idctrow_special(block+16,destU8,flag);
	destU8+=stride;
	idctrow_special(block+24,destU8,flag);
	destU8+=stride;
	idctrow_special(block+32,destU8,flag);
	destU8+=stride;
	idctrow_special(block+40,destU8,flag);
	destU8+=stride;
	idctrow_special(block+48,destU8,flag);
	destU8+=stride;
	idctrow_special(block+56,destU8,flag);
	clearblock(block);	


}



/* init common structure for both encoder and decoder */
int MPV_common_init(MpegEncContext *s)
{
    int c_size, i;
    UINT8 *pict;

    s->mb_width = (s->width + 15) / 16;
    s->mb_height = (s->height + 15) / 16;
    s->linesize = s->mb_width * 16 + 2 * EDGE_WIDTH;

    for(i=0;i<3;i++) {
        int w, h, shift, pict_start;

        w = s->linesize;
        h = s->mb_height * 16 + 2 * EDGE_WIDTH;
        shift = (i == 0) ? 0 : 1;
        c_size = (w >> shift) * (h >> shift);
        pict_start = (w >> shift) * (EDGE_WIDTH >> shift) + (EDGE_WIDTH >> shift);

        pict = av_mallocz(c_size);
        if (pict == NULL)
            goto fail;
        s->last_picture_base[i] = pict;
        s->last_picture[i] = pict + pict_start;
    
        pict = av_mallocz(c_size);
        if (pict == NULL)
            goto fail;
        s->next_picture_base[i] = pict;
        s->next_picture[i] = pict + pict_start;

        if (s->has_b_frames) {
            pict = av_mallocz(c_size);
            if (pict == NULL) 
                goto fail;
            s->aux_picture_base[i] = pict;
            s->aux_picture[i] = pict + pict_start;
        }
    }

    if (s->out_format == FMT_H263) {
        int size;
        /* MV prediction */
        size = (2 * s->mb_width + 2) * (2 * s->mb_height + 2);
        s->motion_val = malloc(size * 2 * sizeof(INT16));
        if (s->motion_val == NULL)
            goto fail;
        memset(s->motion_val, 0, size * 2 * sizeof(INT16));
    }

    if (s->h263_pred) {
        int y_size, c_size, i, size;
        
        /* dc values */

        y_size = (2 * s->mb_width + 2) * (2 * s->mb_height + 2);
        c_size = (s->mb_width + 2) * (s->mb_height + 2);
        size = y_size + 2 * c_size;
        s->dc_val[0] = malloc(size * sizeof(INT16));
        if (s->dc_val[0] == NULL)
            goto fail;
        s->dc_val[1] = s->dc_val[0] + y_size;
        s->dc_val[2] = s->dc_val[1] + c_size;
        for(i=0;i<size;i++)
            s->dc_val[0][i] = 1024;

        /* ac values */
        s->ac_val[0] = av_mallocz(size * sizeof(INT16) * 16);
        if (s->ac_val[0] == NULL)
            goto fail;
        s->ac_val[1] = s->ac_val[0] + y_size;
        s->ac_val[2] = s->ac_val[1] + c_size;
        
        /* cbp values */
        s->coded_block = av_mallocz(y_size);
        if (!s->coded_block)
            goto fail;
    }
    /* default structure is frame */
    s->picture_structure = PICT_FRAME;

    /* init default q matrix (only for mpeg and mjpeg) */
    for(i=0;i<64;i++) {
        s->intra_matrix[i] = default_intra_matrix[i];
        s->chroma_intra_matrix[i] = default_intra_matrix[i];
        s->non_intra_matrix[i] = default_non_intra_matrix[i];
        s->chroma_non_intra_matrix[i] = default_non_intra_matrix[i];
    }
    s->context_initialized = 1;
    return 0;
 fail:
    if (s->motion_val)
        free(s->motion_val);
    if (s->dc_val[0])
        free(s->dc_val[0]);
    if (s->ac_val[0])
        free(s->ac_val[0]);
    if (s->coded_block)
        free(s->coded_block);
    for(i=0;i<3;i++) {
        if (s->last_picture_base[i])
            free(s->last_picture_base[i]);
        if (s->next_picture_base[i])
            free(s->next_picture_base[i]);
        if (s->aux_picture_base[i])
            free(s->aux_picture_base[i]);
    }
    return -1;
}

/* init common structure for both encoder and decoder */
void MPV_common_end(MpegEncContext *s)
{
    int i;

    if (s->motion_val)
        free(s->motion_val);
    if (s->h263_pred) {
        free(s->dc_val[0]);
        free(s->ac_val[0]);
        free(s->coded_block);
    }
    for(i=0;i<3;i++) {
        free(s->last_picture_base[i]);
        free(s->next_picture_base[i]);
        if (s->has_b_frames)
            free(s->aux_picture_base[i]);
    }
    s->context_initialized = 0;
}


/* draw the edges of width 'w' of an image of size width, height */
static void draw_edges(UINT8 *buf, int wrap, int width, int height, int w)
{
    UINT8 *ptr, *last_line;
    int i;

    last_line = buf + (height - 1) * wrap;
    for(i=0;i<w;i++) {
        /* top and bottom */
        memcpy(buf - (i + 1) * wrap, buf, width);
        memcpy(last_line + (i + 1) * wrap, last_line, width);
    }
    /* left and right */
    ptr = buf;
    for(i=0;i<height;i++) {
        memset(ptr - w, ptr[0], w);
        memset(ptr + width, ptr[width-1], w);
        ptr += wrap;
    }
    /* corners */
    for(i=0;i<w;i++) {
        memset(buf - (i + 1) * wrap - w, buf[0], w); /* top left */
        memset(buf - (i + 1) * wrap + width, buf[width-1], w); /* top right */
        memset(last_line + (i + 1) * wrap - w, last_line[0], w); /* top left */
        memset(last_line + (i + 1) * wrap + width, last_line[width-1], w); /* top right */
    }
}


/* generic function for encode/decode called before a frame is coded/decoded */
void MPV_frame_start(MpegEncContext *s)
{
    int i;
    UINT8 *tmp;

    if (s->pict_type == B_TYPE) {
        for(i=0;i<3;i++) {
            s->current_picture[i] = s->aux_picture[i];
        }
    } else {
        for(i=0;i<3;i++) {
            /* swap next and last */
            tmp = s->last_picture[i];
            s->last_picture[i] = s->next_picture[i];
            s->next_picture[i] = tmp;
            s->current_picture[i] = tmp;
        }
    }
}

/* generic function for encode/decode called after a frame has been coded/decoded */
void MPV_frame_end(MpegEncContext *s)
{
    /* draw edge for correct motion prediction if outside */
    if (s->pict_type != B_TYPE) {
        draw_edges(s->current_picture[0], s->linesize, s->width, s->height, EDGE_WIDTH);
        draw_edges(s->current_picture[1], s->linesize/2, s->width/2, s->height/2, EDGE_WIDTH/2);
        draw_edges(s->current_picture[2], s->linesize/2, s->width/2, s->height/2, EDGE_WIDTH/2);
    }
}


static int clip(int a, int amin, int amax)
{
    if (a < amin)
        return amin;
    else if (a > amax)
        return amax;
    else
        return a;
}


/* apply one mpeg motion vector to the three components */
static  void mpeg_motion(MpegEncContext *s,
                               UINT8 *dest_y, UINT8 *dest_cb, UINT8 *dest_cr,
                               int dest_offset,
                               UINT8 **ref_picture, int src_offset,
                               int field_based, op_pixels_func *pix_op,
                               int motion_x, int motion_y, int h)
{
    UINT8 *ptr;
    int dxy, offset, mx, my, src_x, src_y, height, linesize;
    
    dxy = ((motion_y & 1) << 1) | (motion_x & 1);
    src_x = s->mb_x * 16 + (motion_x >> 1);
    src_y = s->mb_y * (16 >> field_based) + (motion_y >> 1);
                
    /* WARNING: do no forget half pels */
    height = s->height >> field_based;
    src_x = clip(src_x, -16, s->width);
    if (src_x == s->width)
        dxy &= ~1;
    src_y = clip(src_y, -16, height);
    if (src_y == height)
        dxy &= ~2;
    linesize = s->linesize << field_based;
    ptr = ref_picture[0] + (src_y * linesize) + (src_x) + src_offset;
    dest_y += dest_offset;
    pix_op[dxy](dest_y, ptr, linesize, h);
    pix_op[dxy](dest_y + 8, ptr + 8, linesize, h);

    if (s->out_format == FMT_H263) {
        dxy = 0;
        if ((motion_x & 3) != 0)
            dxy |= 1;
        if ((motion_y & 3) != 0)
            dxy |= 2;
        mx = motion_x >> 2;
        my = motion_y >> 2;
    } else {
        mx = motion_x / 2;
        my = motion_y / 2;
        dxy = ((my & 1) << 1) | (mx & 1);
        mx >>= 1;
        my >>= 1;
    }
    
    src_x = s->mb_x * 8 + mx;
    src_y = s->mb_y * (8 >> field_based) + my;
    src_x = clip(src_x, -8, s->width >> 1);
    if (src_x == (s->width >> 1))
        dxy &= ~1;
    src_y = clip(src_y, -8, height >> 1);
    if (src_y == (height >> 1))
        dxy &= ~2;

    offset = (src_y * (linesize >> 1)) + src_x + (src_offset >> 1);
    ptr = ref_picture[1] + offset;
    pix_op[dxy](dest_cb + (dest_offset >> 1), ptr, linesize >> 1, h >> 1);
    ptr = ref_picture[2] + offset;
    pix_op[dxy](dest_cr + (dest_offset >> 1), ptr, linesize >> 1, h >> 1);
}

static  void MPV_motion(MpegEncContext *s, 
                              UINT8 *dest_y, UINT8 *dest_cb, UINT8 *dest_cr,
                              int dir, UINT8 **ref_picture, 
                              op_pixels_func *pix_op)
{
    int dxy, offset, mx, my, src_x, src_y, motion_x, motion_y;
    int mb_x, mb_y, i;
    UINT8 *ptr, *dest;

    mb_x = s->mb_x;
    mb_y = s->mb_y;

    switch(s->mv_type) {
    case MV_TYPE_16X16:
        mpeg_motion(s, dest_y, dest_cb, dest_cr, 0,
                    ref_picture, 0,
                    0, pix_op,
                    s->mv[dir][0][0], s->mv[dir][0][1], 16);
        break;
    case MV_TYPE_8X8:
        for(i=0;i<4;i++) {
            motion_x = s->mv[dir][i][0];
            motion_y = s->mv[dir][i][1];

            dxy = ((motion_y & 1) << 1) | (motion_x & 1);
            src_x = mb_x * 16 + (motion_x >> 1) + (i & 1) * 8;
            src_y = mb_y * 16 + (motion_y >> 1) + ((i >> 1) & 1) * 8;
                    
            /* WARNING: do no forget half pels */
            src_x = clip(src_x, -16, s->width);
            if (src_x == s->width)
                dxy &= ~1;
            src_y = clip(src_y, -16, s->height);
            if (src_y == s->height)
                dxy &= ~2;
                    
            ptr = ref_picture[0] + (src_y * s->linesize) + (src_x);
            dest = dest_y + ((i & 1) * 8) + (i >> 1) * 8 * s->linesize;
            pix_op[dxy](dest, ptr, s->linesize, 8);
        }
        /* In case of 8X8, we construct a single chroma motion vector
           with a special rounding */
        mx = 0;
        my = 0;
        for(i=0;i<4;i++) {
            mx += s->mv[dir][i][0];
            my += s->mv[dir][i][1];
        }
        if (mx >= 0)
            mx = (h263_chroma_roundtab[mx & 0xf] + ((mx >> 3) & ~1));
        else {
            mx = -mx;
            mx = -(h263_chroma_roundtab[mx & 0xf] + ((mx >> 3) & ~1));
        }
        if (my >= 0)
            my = (h263_chroma_roundtab[my & 0xf] + ((my >> 3) & ~1));
        else {
            my = -my;
            my = -(h263_chroma_roundtab[my & 0xf] + ((my >> 3) & ~1));
        }
        dxy = ((my & 1) << 1) | (mx & 1);
        mx >>= 1;
        my >>= 1;

        src_x = mb_x * 8 + mx;
        src_y = mb_y * 8 + my;
        src_x = clip(src_x, -8, s->width/2);
        if (src_x == s->width/2)
            dxy &= ~1;
        src_y = clip(src_y, -8, s->height/2);
        if (src_y == s->height/2)
            dxy &= ~2;
        
        offset = (src_y * (s->linesize >> 1)) + src_x;
        ptr = ref_picture[1] + offset;
        pix_op[dxy](dest_cb, ptr, s->linesize >> 1, 8);
        ptr = ref_picture[2] + offset;
        pix_op[dxy](dest_cr, ptr, s->linesize >> 1, 8);
        break;
    case MV_TYPE_FIELD:
        if (s->picture_structure == PICT_FRAME) {
            /* top field */
            mpeg_motion(s, dest_y, dest_cb, dest_cr, 0,
                        ref_picture, s->field_select[dir][0] ? s->linesize : 0,
                        1, pix_op,
                        s->mv[dir][0][0], s->mv[dir][0][1], 8);
            /* bottom field */
            mpeg_motion(s, dest_y, dest_cb, dest_cr, s->linesize,
                        ref_picture, s->field_select[dir][1] ? s->linesize : 0,
                        1, pix_op,
                        s->mv[dir][1][0], s->mv[dir][1][1], 8);
        } else {
            

        }
        break;
    }
}


static void  put_dct(MpegEncContext *s, 
                           DCTELEM *block, int i, UINT8 *dest, int line_size)
{
    dct_unquantize(s, block, i, s->qscale);
	idct_special (block, dest, line_size, 0);
}

/* add block[] to dest[] */
static void  add_dct(MpegEncContext *s, 
                           DCTELEM *block, int i, UINT8 *dest, int line_size)
{
    if (s->block_last_index[i] >= 0) {
		dct_unquantize(s, block, i, s->qscale);
		idct_special (block, dest, line_size, 1);
    }
}

/* generic function called after a macroblock has been parsed by the
   decoder or after it has been encoded by the encoder.

   Important variables used:
   s->mb_intra : true if intra macroblock
   s->mv_dir   : motion vector direction
   s->mv_type  : motion vector type
   s->mv       : motion vector
   s->interlaced_dct : true if interlaced dct used (mpeg2)
 */
void MPV_decode_mb(MpegEncContext *s, DCTELEM block[6][64])
{
    int mb_x, mb_y, motion_x, motion_y;
    int dct_linesize, dct_offset;
    op_pixels_func *op_pix;

    mb_x = s->mb_x;
    mb_y = s->mb_y;

    /* update DC predictors for P macroblocks */
    if (!s->mb_intra) {
        if (s->h263_pred) {
            int wrap, x, y, v;
            wrap = 2 * s->mb_width + 2;
            v = 1024;
            x = 2 * mb_x + 1;
            y = 2 * mb_y + 1;
            s->dc_val[0][(x) + (y) * wrap] = v;
            s->dc_val[0][(x + 1) + (y) * wrap] = v;
            s->dc_val[0][(x) + (y + 1) * wrap] = v;
            s->dc_val[0][(x + 1) + (y + 1) * wrap] = v;
            /* ac pred */
            memset(s->ac_val[0][(x) + (y) * wrap], 0, 16 * sizeof(INT16));
            memset(s->ac_val[0][(x + 1) + (y) * wrap], 0, 16 * sizeof(INT16));
            memset(s->ac_val[0][(x) + (y + 1) * wrap], 0, 16 * sizeof(INT16));
            memset(s->ac_val[0][(x + 1) + (y + 1) * wrap], 0, 16 * sizeof(INT16));
            if (s->h263_msmpeg4) {
                s->coded_block[(x) + (y) * wrap] = 0;
                s->coded_block[(x + 1) + (y) * wrap] = 0;
                s->coded_block[(x) + (y + 1) * wrap] = 0;
                s->coded_block[(x + 1) + (y + 1) * wrap] = 0;
            }
            /* chroma */
            wrap = s->mb_width + 2;
            x = mb_x + 1;
            y = mb_y + 1;
            s->dc_val[1][(x) + (y) * wrap] = v;
            s->dc_val[2][(x) + (y) * wrap] = v;
            /* ac pred */
            memset(s->ac_val[1][(x) + (y) * wrap], 0, 16 * sizeof(INT16));
            memset(s->ac_val[2][(x) + (y) * wrap], 0, 16 * sizeof(INT16));
        } else {
            s->last_dc[0] = 128 << s->intra_dc_precision;
            s->last_dc[1] = 128 << s->intra_dc_precision;
            s->last_dc[2] = 128 << s->intra_dc_precision;
        }
    }
    
    /* update motion predictor */
    if (s->out_format == FMT_H263) {
        int x, y, wrap;
        
        x = 2 * mb_x + 1;
        y = 2 * mb_y + 1;
        wrap = 2 * s->mb_width + 2;
        if (s->mb_intra) {
            motion_x = 0;
            motion_y = 0;
            goto motion_init;
        } else if (s->mv_type == MV_TYPE_16X16) {
            motion_x = s->mv[0][0][0];
            motion_y = s->mv[0][0][1];
        motion_init:
            /* no update if 8X8 because it has been done during parsing */
            s->motion_val[(x) + (y) * wrap][0] = motion_x;
            s->motion_val[(x) + (y) * wrap][1] = motion_y;
            s->motion_val[(x + 1) + (y) * wrap][0] = motion_x;
            s->motion_val[(x + 1) + (y) * wrap][1] = motion_y;
            s->motion_val[(x) + (y + 1) * wrap][0] = motion_x;
            s->motion_val[(x) + (y + 1) * wrap][1] = motion_y;
            s->motion_val[(x + 1) + (y + 1) * wrap][0] = motion_x;
            s->motion_val[(x + 1) + (y + 1) * wrap][1] = motion_y;
        }
    }
    
    if (!s->intra_only) {
        UINT8 *dest_y, *dest_cb, *dest_cr;

        dest_y = s->current_picture[0] + (mb_y * 16 * s->linesize) + mb_x * 16;
        dest_cb = s->current_picture[1] + (mb_y * 8 * (s->linesize >> 1)) + mb_x * 8;
        dest_cr = s->current_picture[2] + (mb_y * 8 * (s->linesize >> 1)) + mb_x * 8;

        if (s->interlaced_dct) {
            dct_linesize = s->linesize * 2;
            dct_offset = s->linesize;
        } else {
            dct_linesize = s->linesize;
            dct_offset = s->linesize * 8;
        }

        if (!s->mb_intra) {
            /* motion handling */
            if (!s->no_rounding) 
                op_pix = put_pixels_tab;
            else
                op_pix = put_no_rnd_pixels_tab;

            if (s->mv_dir & MV_DIR_FORWARD) {
                MPV_motion(s, dest_y, dest_cb, dest_cr, 0, s->last_picture, op_pix);
                if (!s->no_rounding) 
                    op_pix = avg_pixels_tab;
                else
                    op_pix = avg_no_rnd_pixels_tab;
            }
            if (s->mv_dir & MV_DIR_BACKWARD) {
                MPV_motion(s, dest_y, dest_cb, dest_cr, 1, s->next_picture, op_pix);
            }

            /* add dct residue */
            add_dct(s, block[0], 0, dest_y, dct_linesize);
            add_dct(s, block[1], 1, dest_y + 8, dct_linesize);
            add_dct(s, block[2], 2, dest_y + dct_offset, dct_linesize);
            add_dct(s, block[3], 3, dest_y + dct_offset + 8, dct_linesize);

            add_dct(s, block[4], 4, dest_cb, dct_linesize >> 1);
            add_dct(s, block[5], 5, dest_cr, dct_linesize >> 1);
        } else {
            /* dct only in intra block */
            put_dct(s, block[0], 0, dest_y, dct_linesize);
            put_dct(s, block[1], 1, dest_y + 8, dct_linesize);
            put_dct(s, block[2], 2, dest_y + dct_offset, dct_linesize);
            put_dct(s, block[3], 3, dest_y + dct_offset + 8, dct_linesize);

            put_dct(s, block[4], 4, dest_cb, dct_linesize >> 1);
            put_dct(s, block[5], 5, dest_cr, dct_linesize >> 1);
        }
    }
}



static void dct_unquantize(MpegEncContext *s, 
                           DCTELEM *block, int n, int qscale)
{
    int i, level;
    const UINT16 *quant_matrix;

    if (s->mb_intra) {
        if (n < 4) 
            block[0] = block[0] * s->y_dc_scale;
        else
            block[0] = block[0] * s->c_dc_scale;
        if (s->out_format == FMT_H263) {
            i = 1;
            goto unquant_even;
        }
        /* XXX: only mpeg1 */
        quant_matrix = s->intra_matrix;
        for(i=1;i<64;i++) {
            level = block[i];
            if (level) {
                if (level < 0) {
                    level = -level;
                    level = (int)(level * qscale * quant_matrix[i]) >> 3;
                    level = (level - 1) | 1;
                    level = -level;
                } else {
                    level = (int)(level * qscale * quant_matrix[i]) >> 3;
                    level = (level - 1) | 1;
                }
                block[i] = level;
            }
        }
    } else {
        i = 0;
    unquant_even:
        quant_matrix = s->non_intra_matrix;
        for(;i<64;i++) {
            level = block[i];
            if (level) {
                if (level < 0) {
                    level = -level;
                    level = (((level << 1) + 1) * qscale *
                             ((int) (quant_matrix[i]))) >> 4;
                    level = (level - 1) | 1;
                    level = -level;
                } else {
                    level = (((level << 1) + 1) * qscale *
                             ((int) (quant_matrix[i]))) >> 4;
                    level = (level - 1) | 1;
                }
                block[i] = level;
            }
        }
    }
}
                         

/* rate control */

/* an I frame is I_FRAME_SIZE_RATIO bigger than a P frame */
#define I_FRAME_SIZE_RATIO 3.0
#define QSCALE_K           20

static void rate_control_init(MpegEncContext *s)
{
    s->wanted_bits = 0;

    if (s->intra_only) {
        s->I_frame_bits = ((INT64)s->bit_rate * FRAME_RATE_BASE) / s->frame_rate;
        s->P_frame_bits = s->I_frame_bits;
    } else {
        s->P_frame_bits = (int) ((float)(s->gop_size * s->bit_rate) / 
                                 (float)((float)s->frame_rate / FRAME_RATE_BASE * (I_FRAME_SIZE_RATIO + s->gop_size - 1)));
        s->I_frame_bits = (int)(s->P_frame_bits * I_FRAME_SIZE_RATIO);
    }
    
}


/*
 * This heuristic is rather poor, but at least we do not have to
 * change the qscale at every macroblock.
 */
static int rate_estimate_qscale(MpegEncContext *s)
{
    long total_bits = s->total_bits;
    float q;
    int qscale, diff, qmin;

    if (s->pict_type == I_TYPE) {
        s->wanted_bits += s->I_frame_bits;
    } else {
        s->wanted_bits += s->P_frame_bits;
    }
    diff = s->wanted_bits - total_bits;
    q = 31.0 - (float)diff / (QSCALE_K * s->mb_height * s->mb_width);
    /* adjust for I frame */
    if (s->pict_type == I_TYPE && !s->intra_only) {
        q /= I_FRAME_SIZE_RATIO;
    }

    /* using a too small Q scale leeds to problems in mpeg1 and h263
       because AC coefficients are clamped to 255 or 127 */
    qmin = 3;
    if (q < qmin)
        q = qmin;
    else if (q > 31)
        q = 31;
    qscale = (int)(q + 0.5);
    return qscale;
}

